<?php

namespace Drupal\fox\Drush\Commands;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\fox\FoxEntityQuery;
use Drupal\fox\Plugin\FoxCommandsManager;
use Drush\Attributes as CLI;
use Drush\Commands\DrushCommands;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * A Drush commandfile.
 *
 * In addition to this file, you need a drush.services.yml
 * in root of your module, and a composer.json file that provides the name
 * of the services file to use.
 */
class FoxCommands extends DrushCommands {

  /**
   * Options list.
   *
   * @var array
   */
  protected $options;

  /**
   * Variables list.
   *
   * @var array
   */
  protected $variables = [];

  /**
   * Constructs a FoxCommands object.
   */
  public function __construct(
    private readonly EntityTypeManagerInterface $entityTypeManager,
    private readonly FoxCommandsManager $foxCommandsManager,
    private readonly FoxEntityQuery $foxEntityQuery,
  ) {
    parent::__construct();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('plugin.manager.fox_commands'),
      $container->get('fox.query.entity'),
    );
  }

  /**
   * Fox console help.
   */
  #[CLI\Command(name: 'fox:help', aliases: ['foxh'])]
  public function help() {
    $header = [dt('Command'), dt('Description')];
    $data = [];
    $foxCommands = $this->foxCommandsManager->getDefinitions();
    foreach ($foxCommands as $name => $info) {
      $data[] = [strtoupper($name), $info['label']];
    }

    $this->io()->table($header, $data);

    $header = [dt('Query'), dt('Description')];
    $data = [];
    $data[] = [
      dt('Entity query'),
      dt('[FROM entity_type[.bundle]] [WHERE conditions] [ORDER BY orders] [LIMIT limit OFFSET offset]'),
    ];

    $this->io()->table($header, $data);
  }

  /**
   * Fox console.
   */
  #[CLI\Command(name: 'fox:console', aliases: ['foxc'])]
  #[CLI\Option(name: 'mode', description: 'Command mode: default|debug')]
  public function console($options = ['mode' => 'default']) {
    $this->options = $options;
    $this->io()->info(dt("Welcome to the Fox console\nParameters: mode [default]|debug"));

    // Enter the interactive loop.
    while (TRUE) {
      $input = $this->io()->ask(dt('Enter a command (type "exit" to quit): '));

      // Check for exit condition.
      if ($input === 'exit') {
        $this->io()->info('Exiting the Fox console.');
        break;
      }

      // All commands.
      $commands = array_filter(explode(';', trim($input)));
      $last_key = end(array_keys($commands));
      foreach ($commands as $key => $command) {
        $command = trim($command);
        $completed = FALSE;

        // Fox commands.
        if (!$completed) {
          $result = $this->parseFoxCommands($command);
          if (!empty($result)) {
            $this->setVariables($result);
            $this->printResult($result);
            $completed = TRUE;
          }
        }

        // Query parsing.
        if (!$completed) {
          $result = $this->foxEntityQuery->doEntityQuery($command, $this->variables, $this->options);
          if (!empty($result)) {
            $this->setVariables($result);
            $this->printResult($result);
            $completed = TRUE;
          }
        }

        // Show variables.
        if ($key == $last_key) {
          $message = '';
          foreach ($this->variables as $name => $value) {
            if ($value) {
              $message .= "$name=\"$value\" ";
            }
          }
          $this->output->writeln($message);
        }
      }
    }
  }

  /**
   * Fox commands parse.
   *
   * @param string $input
   *   User input.
   *
   * @return string
   *   Output message.
   */
  protected function parseFoxCommands(string $input) {
    $foxCommands = $this->foxCommandsManager->getDefinitions();

    $params = array_filter(explode(' ', $input));
    $command = strtolower($params[0]);
    if (!empty($foxCommands[$command])) {
      array_shift($params);
      $instance = $this->foxCommandsManager->createInstance($command);
      $result = $instance->execute($params, $this->variables);
      return $result;
    }

    return NULL;
  }

  /**
   * Print result.
   *
   * @param array $result
   *   Command result.
   */
  protected function printResult(array $result): void {
    if (empty($result)) {
      return;
    }

    if (isset($result['error'])) {
      $this->io()->error($result['error']);
      return;
    }

    if (isset($result['debug'])) {
      $this->io()->info($result['debug']);
    }

    $message = $result['message'] ?? NULL;
    if (!empty($message)) {
      if (is_array($message)) {
        if (empty($message['data'])) {
          $this->io()->warning(dt('Empty data'));
        }
        else {
          $this->io()->table($message['header'], $message['data']);
          $this->output()->writeln(dt('Count: @count', [
            '@count' => count($message['data']),
          ]));
        }
      }
      else {
        $this->output()->writeln($message);
      }
    }

  }

  /**
   * Set local variables.
   *
   * @param array $result
   *   Command result.
   */
  protected function setVariables($result): void {
    $variables = $result['variables'] ?? NULL;
    if (!$variables) {
      return;
    }

    foreach ($variables as $name => $value) {
      $this->variables[$name] = $value;
    }
  }

}
