<?php

declare(strict_types = 1);

namespace Drupal\fox;

use Drupal\Core\StringTranslation\StringTranslationTrait;
use PHPSQLParser\PHPSQLParser;

/**
 * Fox entity query service.
 */
class FoxEntityQuery {

  use StringTranslationTrait;

  /**
   * Constructs a ForQueryEntity object.
   */
  public function __construct(
    private readonly FoxCommandsHelper $foxCommandsHelper,
  ) {}

  /**
   * Do entity query.
   *
   * @param string $input
   *   User input.
   * @param array $variables
   *   Current variables.
   * @param array $options
   *   Options values.
   *
   * @return array
   *   Result data.
   */
  public function doEntityQuery(string $input, array $variables, array $options): array {
    $query_string = $this->foxCommandsHelper->stringRender($input, $variables);

    $parser = new PHPSQLParser($query_string);
    $parsed = $parser->parsed;

    if (empty($parsed)) {
      return [
        'error' => $this->t('Wrong query or command.'),
      ];
    }

    $debug = $options['mode'] === 'debug';

    $entity_type = $variables['entity_type'] ?? NULL;
    $bundle = $variables['bundle'] ?? NULL;

    // FROM clause.
    if (isset($parsed['FROM'])) {
      $from = current($parsed['FROM'])['no_quotes']['parts'];
      [$entity_type, $bundle] = $this->foxCommandsHelper->getTypeBundle($from);
    }
    else {
      if (empty($entity_type)) {
        return [
          'error' => $this->t('Empty entity type'),
        ];
      }
    }

    try {
      $query = $this->foxCommandsHelper->getEntityQuery($entity_type, $bundle);

      // WHERE clause.
      $where = $parsed['WHERE'] ?? NULL;
      if ($where) {
        $groups = $this->parseWhere($where);
        foreach ($groups as $group) {
          $conditionGroup = NULL;
          if ($group['group_operator'] === 'AND') {
            $conditionGroup = $query->andConditionGroup();
          }
          elseif ($group['group_operator'] === 'OR') {
            $conditionGroup = $query->orConditionGroup();
          }
          if (empty($conditionGroup)) {
            continue;
          }

          foreach ($group['conditions'] as $condition) {
            $conditionGroup->condition($condition['field'], $condition['value'], $condition['operator']);
          }
          $query->condition($conditionGroup);
        }
      }

      // ORDER clause.
      $orders = $parsed['ORDER'] ?? NULL;
      if ($orders) {
        foreach ($orders as $order) {
          $query->sort($order['base_expr'], $order['direction']);
        }
      }

      // LIMIT clause.
      $limits = $parsed['LIMIT'] ?? NULL;
      if ($limits) {
        $offset = 0;
        if (is_numeric($limits['offset'])) {
          $offset = $limits['offset'];
        }
        $limit = $limits['rowcount'];
        $query->range($offset, $limit);
      }

      $ids = $query->execute();
    }
    catch (\Exception $e) {
      return [
        'error' => $e->getMessage(),
      ];
    }

    // Format data.
    $header = [$this->t('ID')];
    $data = [];
    foreach ($ids as $id) {
      $data[] = [$id];
    }

    if (!empty($ids)) {
      // SELECT clause.
      $select = $parsed['SELECT'] ?? NULL;
      if ($select) {
        $entities = $this->foxCommandsHelper->entityTypeManager()
          ->getStorage($entity_type)
          ->loadMultiple($ids);

        $data = [];
        foreach ($select as $field) {
          $name = $field['base_expr'];
          if ($name == '*') {
            continue;
          }

          $header[] = ucfirst($name);
        }

        foreach ($entities as $entity) {
          $row = [$entity->id()];
          foreach ($select as $field) {
            $name = $field['base_expr'];
            if (!$entity->hasField($name)) {
              continue;
            }
            $value = $entity->get($name)->getString();
            $row[] = $value;
          }
          $data[] = $row;
        }
      }
    }

    $result = [
      'message' => [
        'header' => $header,
        'data' => $data,
      ],
      'variables' => [
        'entityType' => $entity_type,
        'bundle' => $bundle,
        'count' => NULL,
        'id' => NULL,
      ],
    ];

    if ($debug) {
      $result['debug'] = (string) $query;
    }

    return $result;
  }

  /**
   * Where parsing.
   *
   * @param array $where
   *   Where data.
   *
   * @return array
   *   Condition groups.
   */
  protected function parseWhere(array $where): array {
    $groups = [];
    $current_group_operator = NULL;
    $group_index = -1;
    for ($i = 0; $i < count($where);) {
      if ($where[$i]['expr_type'] == 'colref') {
        $value = $where[$i + 2]['base_expr'];
        if ($where[$i + 2]['expr_type'] === 'in-list') {
          $value = [];
          foreach ($where[$i + 2]['sub_tree'] as $item) {
            $value[] = $item['base_expr'];
          }
        }

        $condition = [
          'field' => $where[$i]['base_expr'],
          'operator' => $where[$i + 1]['base_expr'],
          'value' => $value,
        ];
        $i += 3;
      }

      if ($i >= count($where)) {
        if (empty($current_group_operator)) {
          $group_operator = 'AND';
          $groups[++$group_index] = [
            'group_operator' => $group_operator,
            'conditions' => [0 => $condition],
          ];
        }
        else {
          $groups[$group_index]['conditions'][] = $condition;
        }
        break;
      }

      if ($where[$i]['expr_type'] == 'operator') {
        $group_operator = strtoupper($where[$i]['base_expr']);
        if ($group_operator != $current_group_operator) {
          $groups[++$group_index] = [
            'group_operator' => $group_operator,
            'conditions' => [0 => $condition],
          ];
          $current_group_operator = $group_operator;
        }
        else {
          $groups[$group_index]['conditions'][] = $condition;
        }
      }

      if ($where[$i]['expr_type'] == 'bracket_expression') {
        $sub_tree = $where[$i]['sub_tree'];
        $sub_group = $this->parseWhere($sub_tree);
        $groups = array_merge($groups, $sub_group);
      }

      $i++;
    }

    return $groups;
  }

}
